%define _prefix /usr/local
%define bindir %{_prefix}/bin
%define prgdir %{_prefix}/%{name}
%define is_mandrake %(test -e /etc/mandrake-release && echo 1 || echo 0)
%define is_suse %(test -e /etc/SuSE-release && echo 1 || echo 0)
%define is_fedora %(test -e /etc/fedora-release && echo 1 || echo 0)
%define debug_package %{nil}

# to correctly generate an rpm for other distributions
%define _source_filedigest_algorithm 0
%define _binary_filedigest_algorithm 0
%define _binary_payload w9.gzdio

Name:    flair
Version: 3.1
Release: 6
Vendor:  CERN
Packager: <Vasilis.Vlachoudis@cern.ch>
Prefix:  %{_prefix}
Source:  http://cern.ch/flair/%{name}/%{name}-%{version}-%{release}.tgz
URL:     http://flair.cern
BuildRoot: %{_tmppath}/%{name}-buildroot
License: Free for non-commercial non weapon related use

Summary: flair - an advanced graphical interface for simulation programs (FLUKA, G4...)
Group: Applications/Engineering
Requires: python3
Requires: gnuplot
#%if %is_fedora
Requires: flair-geoviewer
Requires: python3-tkinter
Requires: python3-numpy
Requires: python3-scipy
Requires: python3-pydicom
Requires: python3-matplotlib
Requires: python3-pillow-tk
#%endif
%if %is_suse
#Requires: python3-tk
%endif
#Requires: desktop-file-utils

%description
flair is an advanced user friendly interface for several Monte Carlo
codes. Originally it was developed for FLUKA, where its unique
features strongly contributed to the success of both codes fluka and
flair. From version 3 the interface was separated from the
functionality permitting an easy integration of other simulation
packages.
The current line of development is to make flair a unique
tool for performing calculations with the same input using FLUKA,
Geant4, PENELOPE, and other simulations engines.

Presently is fully supporting the CERN FLUKA http://fluka.cern and
there is ongoing work to support always using the same interactive
experience as before:
- PENELOPE https://www.oecd-nea.org/tools/abstract/detail/nea-1525
- Moira http://cern.ch/moira a Geant4 http://cern.ch/geant4 based
  application, permitting the user to run with Geant4 FLUKA
  equivalent inputs with the same geometry, materials, scoring,...

%prep
rm -rf $RPM_BUILD_ROOT
%setup -q

#%build
#make build

%install
rm -Rf $RPM_BUILD_ROOT
make ROOT=$RPM_BUILD_ROOT install install-bin install-mime
rm $RPM_BUILD_ROOT%{prgdir}/%{name}.spec
rm $RPM_BUILD_ROOT%{prgdir}/makefile

%clean
rm -rf $RPM_BUILD_ROOT

%files
%defattr(-,root,root,0755)
%doc AUTHORS BUGS README LICENSE ChangeLog
%config /%{prgdir}/%{name}.ini
/usr/share/applications/*.desktop
/usr/share/mime/packages/flair.xml
/usr/share/icons/hicolor/48x48/mimetypes/*.png
/usr/share/mimelnk/application/x-fluka.desktop
/usr/share/mimelnk/application/x-flair.desktop
/usr/share/bash-completion/completions/flair
%{prgdir}/AUTHORS
%{prgdir}/BUGS
%{prgdir}/README
%{prgdir}/LICENSE
%{prgdir}/ChangeLog
%{prgdir}/%{name}.manual
%{prgdir}/%{name}
%{prgdir}/%{name}.bat
%{prgdir}/*.py
%{bindir}/*
%{prgdir}/lib/*
%{prgdir}/convert/*
%{prgdir}/deck/*
%{prgdir}/fluka/*
%{prgdir}/moira/*
%{prgdir}/penelope/*
%{prgdir}/__pycache__/*.pyc
%{prgdir}/lib/__pycache__/*.pyc
%{prgdir}/convert/__pycache__/*.pyc
%{prgdir}/deck/__pycache__/*.pyc
%{prgdir}/fluka/__pycache__/*.pyc
%{prgdir}/moira/__pycache__/*.pyc
%{prgdir}/penelope/__pycache__/*.pyc
%{prgdir}/icons/*
%{prgdir}/images/*
%{prgdir}/db/*
%{prgdir}/templates/*
%{prgdir}/examples/*
%{prgdir}/desktop/*

%post
#%update_desktop_database
#%update_mime_database
if [ "$1" = "1" ]; then
	/usr/bin/update-mime-database /usr/share/mime > /dev/null
	if [ -x /usr/bin/gtk-update-icon-cache ]; then
		# I should copy also to crystalsvg!
		# otherwise it doesn't work for Fedora-KDE and CrystalSVG
		/usr/bin/gtk-update-icon-cache --force --ignore-theme-index --quiet /usr/share/icons/hicolor
	fi
	if [ -x /usr/bin/update-desktop-database ]; then
		/usr/bin/update-desktop-database
	fi
fi

%postun
if [ "$1" = "0" ]; then
	/usr/bin/update-mime-database /usr/share/mime > /dev/null
	if [ -x /usr/bin/gtk-update-icon-cache ]; then
		/usr/bin/gtk-update-icon-cache --force --ignore-theme-index --quiet /usr/share/icons/hicolor
	fi
	if [ -x /usr/bin/update-desktop-database ]; then
		/usr/bin/update-desktop-database
	fi
fi
#%clean_desktop_database
#%clean_mime_database

%changelog
* Mon Nov 30 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.1-6
 - Improved geometry parenthesis expansion, with user settable limit
 - Improved geometry optimization between zones
 - Improved performance on geometry color plots by caching colors
 - Automatically generate new definition lists from ini files
 - Automatically cleanup pickle files when sucessfully saved
 - Pickle load is only available as command line argument, otherwise too many
   structures wont be properly initialized
 - Improved 3D visualization
 - Improved refreshing of windows
 - Updated makefiles for new MacOS and python 3.9
 - Better flagging of geomery errors detected during 3D plotting

 Bug fixes:
 - Gnuplot fix error when using formulas on USR-1D
 - HI-PROP corret assertion on Z
 - Corrected Bounding box of rotated BOX bodies
 - GeometryViewer Maximize button show the correct icon
 - GeometryLayers ON/OFF problem fixed
 - InPlaceColor automatically update color when changed
 - Faster scanning of Dicom files
 - relativePath fixed for present directory
 - RTPlan fixed conversion problem from MU -> NP
 - Dicom protections against problematic files

* Thu Oct 08 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.1-5
 - automatically display planner on gnuplot on termination
 - added second axes on plots y2,x2
 Bug fixes:
 - Dicom voxels forced minimum HU to be represented as organ 0
 - Dicom correct filtering when the organ 0 is filtered out
 - Dicom fixed cropping behavior
 - GeometryViewer corrected error label
 - fix COMPOUND layout
 - 3D super sampling out of bounds exception
 - Input cardsSorted force calls to ASSIGNMAT to be returned as list
 - Input cardsSorted when dealing with VOXELS use an adaptive card position
 - Fixed interface problem with icon resizing
 - Corrected unnecessary refresh
 - update input preprocess on card add/delete/clone/paste/...
 - run attach ignore empty directories with a core inside
 - fix orientation cube wrong axis selection

* Mon Sep 05 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.1-4
 - Dicom scan for files include subdirectories with relative names
 Bug fixes:
 - USRBIN to RTDOSE more robust file handling
 - adaptive antialias was using a fixed pixel size
 - geoviewer fix beam divergence display
 - faster response on 3D display during selection
 - 3D image export bug

* Wed Sep 03 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.1-3
 - dicomviewer: improvements in the standalone interface of dicom editor
 - Improved 3D farm renderer for voxel geometries
 - Adaptive antialiasing in 3D
 - Several performance enhancements in 3D
 Bug fixes:
 - DETECT card layout corrected
 - fixed 3D stop latency issue
 - Viewport lines aspect ratio is taken into account
 - Various bug fixes for the dicom and RTViewer
 - fixed Voxel region list when is not embedded in the voxel and input refreshing

* Fri Jul 03 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.1-2
 - USRBIN to DVH: converted to multithreaded
 - RTViewer: added refresh button to force the refreshing/regeneration
   of all needed files
 Bug fixes:
 - RTViewer: properly align USRBIN/RTDOSE with voxel
 - Dicom scan: sort files by location for dicom
 - Dicom reporting: fixed index error
 - Updated download url links

* Fri Jul 03 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.1-1
 - Simplified code of object filling in GeometryEditor
 - Recent files show as disabled or not found, project files that do no longer exist
 - Flag as executable to "usrbin2rtdose.py" program to run as standalone as well
 Bug fixes:
 - Protection for Dicom InstanceNumber which in some dicoms is not a number
 - Insert materials fix undefined variable "mode"
 - ImageOps.colorize() remove the white and blackpoint. Do not exist for pillow
 - remove ftls-model from geoviewer, which was causing problems in some OS

* Tue Jun 30 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.1-0
  New release targeting for medical activities with multiple changes
  in the Dicom handling:
  - New display of the dicom page with a left-hand tree for selecting the
    appropriate item
  - Dicom Editor, that allows to perform simple editing tasks like anonymizing files
    or altering values. Can work on multiple files
  - Select multiple slices show common fields
  - Save modified slices
  - Import/Export of RTDOSE to USRBIN and to DVH directly from the Dicom page
  - usrbin2dvh improved tool, making stochastic re-sampling if the binning is not
    matching the voxel dimensions
  - usrbin2rtdose.py: tool to convert USRBIN files to RTDOSE matching a specific RTPLAN
    Can run as standalone, invoked from Run->Data tab or from the Dicom Page (Import)
  - Improved dicom slice viewer
  - Voxel generation:
    - Uses materials from the internal flair material database
    - Filtering for ROIS. The user can input expressions to provide specific
      materials on ROIS based on CSG expressions
      e.g. VACUUM on -Skin or Steel on +Support -Skin
  - RTViewer multiple improvements:
    - interface changed, integrated in the RTDOSE modality
    - display of ROI
    - automatic creation and use of DVH histograms for both RTDOSE & USRBIN
    - faster display
  - RTPlan improvements:
    - interface changed
    - accept both Ion and Photon plan (no MLC support for the moment)
    - create defines for the automatic normalization of the plan
    - all generated cards are flagged with rtplan and cleaned automatically if
      needed
    RTDOSE:
    - Import/export to USRBIN/DVH
    - Add USRBIN cards with the associated transformations for scoring in the
      input

  Geometry Editor:
  - FARM inclusion: Flair Advanced Render module (C.Theis)
    a high performant 3D renderer for generating high quality photorealistic
    3D geometry plots directly in the geometry editor, comparable to the best 3D renders.
  - Planner tool: creation of RP intervention planning and dose estimation
    directly in the geometry editor, along a time spline path, assuming
    a point operator
  - Movie tool: creation of 3D movies with the use of a camera moving along a
    time-spline path
  - Advanced parenthesis expansion with online-optimization of unwanted zones
    Far more performant with the past and with the in-built fluka one.
  - Multiple orientation systems:
      Y-up (default of flair)
      X-up, Z-up, dicom biped, dicom quadruped

  Input Editor:
  - Considerable speed up on undo/redo actions

Other improvements:
  - Cleanup of project file from default values
  - Cleanup of unneeded update_idletasks() leading to concurrency problems
  - Various Performance enhancements
  - Gy as a unit addition
  - error display messages when external programs are missing
  - Make more readable the code using as much as possible f-strings
  - Printer detection in mac systems

Bug corrections:
  - fixed handling of #include card
  - checking the timestamp of #include files
  - executable setting correction
  - duplicating empty lines in REGIONS
  - keyboard handling with empty inputs
  - fix updating of tabs during docking/undocking of pages
  - Several small bug corrections

* Thu May 21 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-12
- Geometry Editor:
  - faster loading of regions
  - more advanced parenthesis expansion and optimization
Bug corrections
- USRYIELD entering particle was wrongly calculated
- Orientation cube was mis-compiling for v7 and older gnuc++
- parameter evaluation on recursive calls
- f-string corrections

* Mon May 18 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-11
Geometry:
- Volume and area calculation at the same time (new algorithm)
- Zone parsing, optimization and expansion from C++ faster
- Faster loading of geometry
Project:
- Store executable name in the run
Dicom:
- Correct cropping for RTSTRUCT
Various bug fixes:
- Protection in redo
- f-string corrections
- unicode decoding errors now are replaced with a *

* Thu Apr 16 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-10
- Geometry Userdump: activate the particle color and energy range selection
- Accept international characters for filenames and directories.
Various bug fixes
- ZoneTool: do not swap action with 'd' if the dialog is not visible
- Userdump/Image layer: ignore ALPHA flagged antialiased pixels
- Geometry Plot: fixed run selection
- RTViewer: apply normalization only on calculated dose
- GeometryEditor: selection bug
- Usr1D: multiple editing corrected
- Improved Matrix re-normalization accuracy
- QUA bounding box corrected
- Fixed clipping and projection body seg.fault when adding new bodies in some cases

* Wed Mar 25 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-9
- Improved Dicom RTPLAN and RTVIEWER to be compatible with the future fluka release
- VOXEL treatment as body inside the GeometryEditor
- Code refactoring for numpy and pydicom
- synchronize canvas on loading
Bug corrections:
- usrbin2dvh check correctly for depedencies
- Card #define writing as is inside the .flair file, while evaluated
  inside the .inp

* Mon Mar 23 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-8
- Dicom improvements

* Tue Feb 25 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-7
- Bug fixes

* Sat Feb 15 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-6
Improvements:
- GeometryViewer more robust conic determination
- Input doesn't have a filename it uses the same from Project.
- Geometry Editor addes parenthesis expansion and zone cleanup actions
- Added Select All, None, Invert as ribbon buttons in all tabs
- Unified adaptive animate loop for all geometry view ports
- Added camera and basic spline objects
- Improved refresh messages
- Input save button, saves always the project + input
Bug corrections
- Fix conic precision error
- FileDialog fix on invalid window name
- FIX/FREE correction when loading from inactive branches
- Dicom correction bytes vs str
- Restored mime icons in deb packages

* Tue Jan 21 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-5
Various bug corrections

* Tue Jan 21 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-4
Various bug corrections

* Fri Jan 17 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-3
Bug corrections
- Detach correctly the run from the flair program. Redirect the output of rfluka to a input.out
  and regularly inspect the file
- Fix RESNUCLEi data processing
- Fix md5 calculation individually for each include
- Fix #include discrepancies when the input file is externally modified
- Integer number with 10 digits are written in exponential form, to be accepted by FLUKA in
  the fixed format
- Fix bug on importing V2 flair UsrInfo with additional files
- Fix bug on importing V2 flair USRBIN plots
- Fix empty comments bug
- Fix bug on the detection of the plotting axis when U-V was supplied
- Correct manual location detection when running 'fm'
- Better checking of LOW-NEUT card

* Thu Jan 2 2020 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-2
Bug corrections
- FileDialog correction for windows removed hack, and restore properly window dimensions
- REGION plot correction
- Infinite loop InputPage when updating plot trying to save in InputEditor
- Windows CLIPBOARD/PRIMARY error on drag & drop
- Attribute Editing permit entering empty values for transformation

* Fri Dec 20 2019 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-1
- Mainly bug corrections

* Mon Sep 02 2019 Vasilis Vlachoudis <Vasilis.Vlachoudis@cern.ch>
+ Version: 3.0-0
flair v3: This update represents a major version change where
the interface is not depending of the simulation engine (fluka in the past).
- The whole code is being re-written for python v3
- The functionality of fluka is separated from the interface, creating
  abstract interfaces to be able to accommodate various simulation engines.
- Moira simulation is introduced and is able to import fluka geometries, still
  in an experimental level.
- All Fluka calls is compatible only to the CERN fluka structure
- Prototype work is on going to have Moira, a Geant4 simulation application
  to be able to run with the same input provided by flair
- The .flair file now is the master and contains the whole input.
  For backwards compatibility with fluka and md5 sum is kept and if the input
  is changed with external editor it will prefer the new input vs the .flair
  It will be contrallable with a flag in the future
- Volume calculator by C.Theis now is available on the interface, spawning a
  separate jobs/taks.
- The interactive navigator cube is introduced in the geometry editor for easier
  orienting in the 3D system
- The tools in the geometry editor now show a dialog for easier human
  interaction
- The Input editor now is based on the Text and no longer on the Canvas widget
  with drastic speed improvements and additional functionality when working with
  big input files.
  - Multiple selection of cards or groups of cards with Ctrl-Click
  - Search and replace is finally working
  - Accept cards from other simulation packages at the same time
- When writing the input file all child-runs create a unique monolithic input
  with all included embeded. This solves the problem of changes of the input
  files during a run, resulting in problematic runs.
- Gdml is further enhanced and directly importing files into moira format
- Multiple bug fixes
- Performance improvements
